@@ -8,9 +8,10 @@ use 5.010000;
 use Exporter qw( import );
 our @EXPORT_OK = qw( add_entry_to_info_hash json_to_hash );
 
-use JSON qw( decode_json );
+use JSON               qw( decode_json );
+use Term::Choose::Util qw( insert_sep );
 
-use App::YTDL::Helper qw( sec_to_time insert_sep );
+use App::YTDL::Helper qw( sec_to_time );
 
 
 
@@ -70,8 +71,9 @@ sub json_to_hash {
     my ( $opt, $tmp, $json ) = @_;
     my $h_ref = decode_json( $json );
     my $formats = {};
+    my $c = 1;
     for my $format ( @{$h_ref->{formats}} ) {
-        my $fmt = $format->{format_id};
+        my $fmt = $format->{format_id} // 'unkown_fmt_' . $c++;
         $formats->{$fmt}{ext}         = $format->{ext};
         $formats->{$fmt}{format}      = $format->{format};
         $formats->{$fmt}{format_note} = $format->{format_note};
@@ -111,13 +113,18 @@ sub json_to_hash {
 
 sub _prepare_info_hash {
     my ( $opt, $info, $ex, $video_id ) = @_;
-    if ( $info->{$ex}{$video_id}{duration} =~ /^[0-9]+\z/ ) {
-        $info->{$ex}{$video_id}{duration} = sec_to_time( $info->{$ex}{$video_id}{duration}, 1 );
-    }
     $info->{$ex}{$video_id}{title}           ||= time();
     $info->{$ex}{$video_id}{duration}        ||= '-:--:--';
     $info->{$ex}{$video_id}{upload_date}     ||= '0000-00-00';
     $info->{$ex}{$video_id}{upload_datetime} ||= '0000-00-00T00:00:00';
+    $info->{$ex}{$video_id}{upload_id}   //= '';
+    $info->{$ex}{$video_id}{playlist_id} //= '';
+    $info->{$ex}{$video_id}{uploader}    //= $info->{$ex}{$video_id}{uploader_id} // '';
+    #$info->{$ex}{$video_id}{extractor_key} //= $info->{$ex}{$video_id}{extractor} // '';
+
+    if ( $info->{$ex}{$video_id}{duration} =~ /^[0-9]+\z/ ) {
+        $info->{$ex}{$video_id}{duration} = sec_to_time( $info->{$ex}{$video_id}{duration}, 1 );
+    }
     if ( $info->{$ex}{$video_id}{like_count} && $info->{$ex}{$video_id}{dislike_count} ) {
         $info->{$ex}{$video_id}{raters}     ||= $info->{$ex}{$video_id}{like_count} + $info->{$ex}{$video_id}{dislike_count};
         $info->{$ex}{$video_id}{avg_rating} ||= $info->{$ex}{$video_id}{like_count} * 5 / $info->{$ex}{$video_id}{raters};
@@ -135,8 +142,6 @@ sub _prepare_info_hash {
             $opt->{view_count_length} = length $info->{$ex}{$video_id}{view_count}; #
         }
     }
-    $info->{$ex}{$video_id}{uploader}      //= $info->{$ex}{$video_id}{uploader_id};
-    #$info->{$ex}{$video_id}{extractor_key} //= $info->{$ex}{$video_id}{extractor} // '';
 }
 
 
