@@ -12,15 +12,16 @@ use Fcntl          qw( LOCK_EX SEEK_END );
 use File::Basename qw( basename );
 use Time::HiRes    qw( gettimeofday tv_interval );
 
-use Encode::Locale   qw();
-use LWP::UserAgent   qw();
-use Term::ANSIScreen qw( :cursor :screen );
-use Try::Tiny        qw( try catch );
+use Encode::Locale     qw();
+use LWP::UserAgent     qw();
+use Term::ANSIScreen   qw( :cursor :screen );
+use Term::Choose::Util qw( insert_sep );
+use Try::Tiny          qw( try catch );
 
 use if $^O eq 'MSWin32', 'Win32::Console::ANSI';
 
 use App::YTDL::Data   qw( get_new_video_url );
-use App::YTDL::Helper qw( sec_to_time insert_sep encode_fs encode_stdout timestamp_to_upload_date HIDE_CURSOR SHOW_CURSOR );
+use App::YTDL::Helper qw( sec_to_time encode_fs encode_stdout timestamp_to_upload_date HIDE_CURSOR SHOW_CURSOR );
 
 END { print SHOW_CURSOR }
 
@@ -38,7 +39,7 @@ sub download_youtube {
                         unlink encode_fs( $file_name ) or die $!
                     }
                     _download_video( $opt, $info, $ex, $video_id, $fmt, $nr );
-                    if ( $opt->{upload_date_is_file_date} ) {
+                    if ( $opt->{modify_timestamp} ) {
                         timestamp_to_upload_date( $info, $ex, $video_id, $file_name );
                     }
                 }
@@ -268,11 +269,13 @@ sub _return_callback {
 sub _log_info {
     my ( $opt, $info, $ex, $video_id, $fmt ) = @_;
     my ( $sec, $min, $hour, $mday, $mon, $year ) = localtime;
+    my $uploader_id = $info->{$ex}{$video_id}{uploader_id};
+    my $playlist_id = $info->{$ex}{$video_id}{playlist_id};
     my $id = $ex eq 'youtube' ? $video_id : $ex . ' ' . $video_id; #
     my $log_str = sprintf( "%04d-%02d-%02d %02d:%02d", $year + 1900, $mon + 1, $mday, $hour, $min )
         . '>  ' . sprintf( "%11s", $id )
-        . ' | ' . ( $info->{$ex}{$video_id}{uploader_id} // 'xxxxx' )
-        . ' | ' . ( $info->{$ex}{$video_id}{playlist_id} // '-----' )
+        . ' | ' . ( length $uploader_id ? $uploader_id : 'xxxxx' )
+        . ' | ' . ( length $playlist_id ? $playlist_id : '-----' )
         . ' | ' . ( $info->{$ex}{$video_id}{upload_date} )
         . '   ' . basename( $info->{$ex}{$video_id}{file_name}{$fmt} );
     open my $log, '>>:encoding(UTF-8)', encode_fs( $opt->{log_file} ) or die $!;
