@@ -6,9 +6,8 @@ use strict;
 use 5.010000;
 
 use Exporter qw( import );
-our @EXPORT_OK = qw( term_size sec_to_time insert_sep choose_a_dir choose_a_number print_hash get_filename
-                     timestamp_to_upload_date encode_fs encode_stdout_lax encode_stdout write_json read_json
-                     sanitize_for_path HIDE_CURSOR SHOW_CURSOR );
+our @EXPORT_OK = qw( sec_to_time get_filename timestamp_to_upload_date encode_fs encode_stdout_lax encode_stdout
+                    write_json read_json sanitize_for_path HIDE_CURSOR SHOW_CURSOR );
 
 use Encode             qw( encode );
 use Time::Local        qw( timelocal );
@@ -17,7 +16,7 @@ use Unicode::Normalize qw( NFC );
 use Encode::Locale     qw();
 use File::Touch        qw();
 use JSON               qw();
-use Term::Choose::Util qw( print_hash choose_a_dir choose_a_number term_size insert_sep unicode_trim );
+use Term::Choose::Util qw( unicode_trim );
 use Unicode::GCString  qw();
 
 
