@@ -205,9 +205,12 @@ sub _sticky_name_and_url {
 
 sub add_channel_to_history {
     my ( $opt, $info, $ex, $video_id ) = @_;
-    my $channel       = $info->{$ex}{$video_id}{uploader};
-    my $channel_id    = $info->{$ex}{$video_id}{uploader_id};
+    my $channel    = $info->{$ex}{$video_id}{uploader};
+    my $channel_id = $info->{$ex}{$video_id}{uploader_id};
     #my $extractor_key = $info->{$ex}{$video_id}{extractor_key};
+    if ( ! length $channel_id ) {
+        return;
+    }
     my $channel_url;
     $channel_url = 'c#'                if $ex eq 'youtube';
     $channel_url = 'http://vimeo.com/' if $ex eq 'vimeo'; #
@@ -215,7 +218,7 @@ sub add_channel_to_history {
         $channel_url .= $channel_id;
         if ( none{ $channel_url eq $_ } keys %{$opt->{channel_sticky}} ) {
             $opt->{channel_history}{$channel_url} = {
-                channel_url => $channel_url,
+                channel_url => $channel_url, #
                 channel     => $channel,
                 channel_id  => $channel_id,
                 extractor   => $ex,
