@@ -10,14 +10,15 @@ our @EXPORT_OK = qw( get_download_infos );
 
 use File::Spec::Functions qw( catfile catdir );
 
-use List::MoreUtils   qw( none first_index );
-use Term::ANSIScreen  qw( :cursor :screen );
-use Term::Choose      qw( choose );
+use List::MoreUtils    qw( none first_index );
+use Term::ANSIScreen   qw( :cursor :screen );
+use Term::Choose       qw( choose );
+use Term::Choose::Util qw( term_size );
 
 use if $^O eq 'MSWin32', 'Win32::Console::ANSI';
 
 use App::YTDL::Info_Print qw( linefolded_print_info );
-use App::YTDL::Helper     qw( get_filename term_size encode_stdout_lax encode_fs read_json write_json sanitize_for_path );
+use App::YTDL::Helper     qw( get_filename encode_stdout_lax encode_fs read_json write_json sanitize_for_path );
 use App::YTDL::History    qw( add_channel_to_history write_channel_history_to_file );
 
 
@@ -33,10 +34,10 @@ sub get_download_infos {
 
     EXTRACTOR: for my $ex ( sort keys %$info ) { # $b cmp $a ?
         my @video_ids = sort {
-               ( $info->{$ex}{$a}{playlist_id}   // '' ) cmp ( $info->{$ex}{$b}{playlist_id}   // '' )
-            || ( $info->{$ex}{$a}{uploader_id}   // '' ) cmp ( $info->{$ex}{$b}{uploader_id}   // '' )
-            || ( $info->{$ex}{$a}{upload_date}   // '' ) cmp ( $info->{$ex}{$b}{upload_date}   // '' )
-            || ( $info->{$ex}{$a}{title}         // '' ) cmp ( $info->{$ex}{$b}{title}         // '' )
+               $info->{$ex}{$a}{playlist_id} cmp $info->{$ex}{$b}{playlist_id}
+            || $info->{$ex}{$a}{uploader_id} cmp $info->{$ex}{$b}{uploader_id}
+            || $info->{$ex}{$a}{upload_date} cmp $info->{$ex}{$b}{upload_date}
+            || $info->{$ex}{$a}{title}       cmp $info->{$ex}{$b}{title}
         } keys %{$info->{$ex}};
         my $fmt_list;
         my $count = 0;
@@ -113,7 +114,7 @@ sub get_download_infos {
                 mkdir encode_fs( $video_dir ) or die $! if ! -d encode_fs( $video_dir );
             }
             if ( $opt->{channel_dir} == 2 || $opt->{channel_dir} == 1 && $info->{$ex}{$video_id}{from_list} ) {
-                if ( $info->{$ex}{$video_id}{uploader} ) {
+                if ( length $info->{$ex}{$video_id}{uploader} ) {
                     my $channel_name = sanitize_for_path( $info->{$ex}{$video_id}{uploader} );
                     $video_dir = catdir $video_dir, $channel_name;
                     mkdir encode_fs( $video_dir ) or die $! if ! -d encode_fs( $video_dir );
@@ -138,7 +139,7 @@ sub get_download_infos {
                 $opt->{total_nr_videos}++;
 
             }
-            if ( $opt->{max_channels} && $info->{$ex}{$video_id}{uploader_id} ) {
+            if ( $opt->{max_channels} ) {
                 add_channel_to_history( $opt, $info, $ex, $video_id );
             }
         }
@@ -182,8 +183,8 @@ sub _set_preferred_qualities {
     CHOICE: while ( 1 ) {
         my @pre = ( undef );
         my @choices = map {
-            sprintf '%d. choice: %s', $_, join( ', ', map { $_ // '' } @{$pref_qual->{$ex}[$_-1]} )
-        } 1 .. 5;
+            sprintf '%d. choice: %s', $_, join ', ', map( $_ // '', @{$pref_qual->{$ex}[$_-1]} )
+        } 1 .. $opt->{pref_qual_slots};
         my $idx_choice = choose(
             [ @pre, @choices ],
             { prompt => "Set preferred qualities ($ex):", layout => 3, index => 1, undef => '<<' }
@@ -217,10 +218,11 @@ sub _fmt_quality {
     my ( $opt, $info, $fmt_list, $ex, $video_id ) = @_;
     my $auto_quality = $opt->{auto_quality};
     my $fmt_ok;
-    if ( $auto_quality == 0 ) {
-    }
-    elsif ( $auto_quality == 1 && $info->{$ex}{$video_id}{from_list} ) {
-        my $list_id = $info->{$ex}{$video_id}{playlist_id} // $info->{$ex}{$video_id}{uploader_id};
+    if ( $auto_quality eq 'keep_list_channel' && $info->{$ex}{$video_id}{from_list} ) {
+        my $list_id = $info->{$ex}{$video_id}{playlist_id};
+        if ( ! length $list_id ) {
+            $list_id = $info->{$ex}{$video_id}{uploader_id};
+        }
         if ( $list_id ) {
             if ( ! defined $opt->{$list_id} ) {
                 $fmt_list = _choose_fmt( $opt, $info, $ex, $video_id );
@@ -233,7 +235,7 @@ sub _fmt_quality {
             $fmt_ok = 1;
         }
     }
-    elsif ( $auto_quality == 2 ) {
+    elsif ( $auto_quality eq 'keep_extractor' ) {
         if ( ! defined $opt->{$ex} ) {
             $fmt_list = _choose_fmt( $opt, $info, $ex, $video_id );
             return if ! defined $fmt_list;
@@ -245,7 +247,7 @@ sub _fmt_quality {
         $fmt_ok = 1;
 
     }
-    elsif ( $auto_quality == 3 ) {
+    elsif ( $auto_quality eq 'preferred' ) {
         my $pref_qual = read_json( $opt->{preferred_file} );
         my @pref_qualities = @{$pref_qual->{$ex}};
         PQ: for my $pq ( @pref_qualities ) {
@@ -269,7 +271,7 @@ sub _fmt_quality {
             $opt->{up}++;
         }
     }
-    elsif ( $auto_quality == 4 && defined $info->{$ex}{$video_id}{format_id} ) {
+    elsif ( $auto_quality eq 'default' && defined $info->{$ex}{$video_id}{format_id} ) {
         $fmt_list = [ $info->{$ex}{$video_id}{format_id} ];
         $fmt_ok = 1;
     }
