@@ -10,14 +10,14 @@ our @EXPORT_OK = qw( linefolded_print_info );
 
 use Encode qw( decode_utf8 );
 
-use Term::ANSIScreen qw( :cursor :screen );
-use Text::LineFold   qw();
+use Term::ANSIScreen   qw( :cursor :screen );
+use Term::Choose::Util qw( term_size );
+use Text::LineFold     qw();
 
 use if $^O eq 'MSWin32', 'Win32::Console::ANSI';
 
 use App::YTDL::Data         qw( get_download_info_as_json );
 use App::YTDL::Data_Extract qw( json_to_hash );
-use App::YTDL::Helper       qw( term_size );
 
 
 
@@ -36,7 +36,7 @@ sub _prepare_print_info {
     }
     $info->{$ex}{$video_id}{published} = $info->{$ex}{$video_id}{upload_date};
     $info->{$ex}{$video_id}{author}    = $info->{$ex}{$video_id}{uploader};
-    if ( $info->{$ex}{$video_id}{author} && $info->{$ex}{$video_id}{uploader_id} ) {
+    if ( length $info->{$ex}{$video_id}{author} && length $info->{$ex}{$video_id}{uploader_id} ) {
         if ( $info->{$ex}{$video_id}{author} ne $info->{$ex}{$video_id}{uploader_id} ) {
             $info->{$ex}{$video_id}{author} .= ' (' . $info->{$ex}{$video_id}{uploader_id} . ')';
         }
