@@ -18,12 +18,12 @@ use File::Which            qw( which );
 use IPC::System::Simple    qw( capture );
 use List::MoreUtils        qw( first_index );
 use Term::Choose           qw( choose );
+use Term::Choose::Util     qw( print_hash choose_a_dir choose_a_number insert_sep );
 use Term::ReadLine::Simple qw();
 use Text::LineFold         qw();
 use Try::Tiny              qw( try catch );
 
-use App::YTDL::Helper qw( term_size print_hash encode_fs choose_a_dir choose_a_number
-                          insert_sep write_json read_json HIDE_CURSOR SHOW_CURSOR );
+use App::YTDL::Helper qw( encode_fs write_json read_json HIDE_CURSOR SHOW_CURSOR );
 
 
 
@@ -50,14 +50,15 @@ sub _menus {
         ],
         group_merge => [
             [ 'merge_enabled',    "- Enable Merge"     ],
-            [ 'merge_overwrite',  "- Overwrite merged" ],
-            [ 'merge_format',     "- Output formats"   ],
+            [ 'merge_overwrite',  "- ffmpeg overwrites" ],
+            [ 'merge_ext',        "- Output formats"   ],
             [ 'merge_loglevel',   "- Verbosity"        ],
             [ 'merged_in_files',  "- Input files"      ],
         ],
         group_quality => [
             [ 'auto_quality',    "- Auto quality mode"   ],
             [ 'print_pref_qual', "- Preferred qualities" ],
+            [ 'pref_qual_slots', "- Slots 'preferred qualilties'"    ],
         ],
         group_history => [
             [ 'log_info',             "- Logging"         ],
@@ -70,12 +71,12 @@ sub _menus {
             [ 'channel_dir',   "- Channel directory"    ],
         ],
         group_file => [
-            [ 'max_len_f_name',           "- Max filename length" ],
-            [ 'upload_date_is_file_date', "- File timestamp"      ],
+            [ 'max_len_f_name',   "- Max filename length" ],
+            [ 'modify_timestamp', "- File timestamp"      ],
         ],
         group_list_menu => [
             [ 'list_size_idx',   "- List size" ],
-            [ 'sort_item',       "- Sort order"      ],
+            [ 'list_sort_item',  "- Sort order"      ],
             [ 'show_view_count', "- View count" ],
         ],
         group_output => [
@@ -87,6 +88,22 @@ sub _menus {
 }
 
 
+sub _group_prompt {
+    my ( $group ) = @_;
+    my $group_prompt ={
+        group_directory => "Directories:",
+        group_file      => "Files:",
+        group_quality   => "Quality:",
+        group_download  => "Download:",
+        group_merge     => "Merge:",
+        group_history   => "History:",
+        group_list_menu => "List menu:",
+        group_output    => "Appearance:",
+    };
+    return $group_prompt->{$group};
+}
+
+
 sub set_options {
     my ( $opt ) = @_;
     my $menus = _menus();
@@ -107,10 +124,11 @@ sub set_options {
             my $continue = '  CONTINUE';
             my @pre  = ( undef, $group eq 'main' ? $continue : () );
             my @real = map( $_->[1], @$menu );
+            my $prompt = _group_prompt( $group ) // 'Options:';
             # Choose
             my $idx = choose(
                 [ @pre, @real ],
-                { prompt => "Options:", layout => 3, index => 1, clear_screen => 1, undef => $back, default => $old_idx }
+                { prompt => $prompt, layout => 3, index => 1, clear_screen => 1, undef => $back, default => $old_idx }
             );
             if ( ! $idx ) {
                 if ( $group eq 'main' ) {
@@ -147,26 +165,50 @@ sub set_options {
                 pod2usage( { -exitval => 'NOEXIT', -verbose => 2 } );
             }
             elsif ( $choice eq "show_info" ) {
-                my $youtube_dl_version;
+                my ( $youtube_dl_version, $ffmpeg_version, $ffprobe_version );
                 try {
-                    my $youtube_dl = which( 'youtube-dl' ) // 'youtube-dl';
+                    my $youtube_dl = which( 'youtube-dl' );
                     $youtube_dl_version = capture( $youtube_dl, '--version' );
                 };
+                try {
+                    my $ffmpeg = which( 'ffmpeg' );
+                    $ffmpeg_version = capture( $ffmpeg, '-version' );
+                    if ( $ffmpeg_version =~ /^ffmpeg version (\S+)/m ) {
+                        $ffmpeg_version = $1;
+                    }
+                    else {
+                        $ffmpeg_version = '';
+                    }
+                };
+                try {
+                    my $ffprobe = which( 'ffprobe' );
+                    $ffprobe_version = capture( $ffprobe, '-version' );
+                    if ( $ffprobe_version =~ /^ffprobe version (\S+)/m ) {
+                        $ffprobe_version = $1;
+                    }
+                    else {
+                        $ffprobe_version = '';
+                    }
+                };
                 my $version     = '  version ';
                 my $bin         = '    bin   ';
                 my $video_dir   = ' video dir';
                 my $config_dir  = 'config dir';
                 my $youtube_dl  = 'youtube-dl';
+                my $ffmpeg      = '  ffmpeg  ';
+                my $ffprobe     = '  ffprobe ';
                 my $path = {
                     $version    => $main::VERSION,
                     $bin        => catfile( $RealBin, $RealScript ),
                     $video_dir  => $opt->{video_dir},
                     $config_dir => $opt->{config_dir},
-                    $youtube_dl => $youtube_dl_version,
-
+                    $youtube_dl => $youtube_dl_version // 'error',
+                    $ffmpeg     => $ffmpeg_version,
+                    $ffprobe    => $ffprobe_version,
                 };
-                my $keys = [ $bin, $version, $video_dir, $config_dir ];
-                push @$keys, $youtube_dl if $youtube_dl_version;
+                my $keys = [ $bin, $version, $video_dir, $config_dir, $youtube_dl ];
+                push @$keys, $ffmpeg  if $ffmpeg_version;
+                push @$keys, $ffprobe if $ffprobe_version;
                 print_hash( $path, { keys => $keys, preface => ' Close with ENTER' } );
             }
             elsif ( $choice eq "list_extractors" ) {
@@ -189,29 +231,13 @@ sub set_options {
                     );
                 }
             }
-            elsif ( $choice eq "print_pref_qual" ) {
-                my $pref = read_json( $opt->{preferred_file} );
-                if ( ! %$pref ) {
-                    choose( [ 'Close with ENTER' ], { prompt => 'No preferred qualities set.' } );
-                }
-                else {
-                    my @p;
-                    push @p, ' ';
-                    for my $key ( sort keys %$pref ) {
-                        push @p, "$key:";
-                        my $c = 1;
-                        for my $line ( @{$pref->{$key}} ) {
-                            push @p, sprintf "%4d. %s\n", $c, '[' . join( ',', @$line ) . ']';
-                            $c++;
-                        }
-                        push @p, ' ';
-                    }
-                    choose( \@p, { prompt => 'Close with ENTER', layout => 3, justify => 0 } );
-                }
-            }
+
             elsif ( $choice eq "useragent" ) {
-                my $prompt = 'Set the UserAgent: ';
-                _local_read_line( $opt, $choice, $prompt );
+                my $prompt = 'Set the UserAgent.';
+                my $list = [
+                    [ 'UserAgent', $opt->{useragent} ]
+                ];
+                _local_readline( $opt, $choice, $prompt, $list );
                 $opt->{useragent} = 'Mozilla/5.0' if $opt->{useragent} eq '';
                 $opt->{useragent} = ''            if $opt->{useragent} eq '""';
             }
@@ -220,34 +246,18 @@ sub set_options {
                 my $list = [ qw( NO YES ) ];
                 _opt_choose_from_list_idx( $opt, $choice, $prompt, $list );
             }
-            elsif ( $choice eq "log_info" ) {
-                my $prompt = 'Enable info-logging';
-                my $list = [ qw( NO YES ) ];
-                _opt_choose_from_list_idx( $opt, $choice, $prompt, $list );
-            }
-            elsif ( $choice eq "auto_quality" ) {
-                my $prompt = 'Auto quality';
-                my $list = [
-                    'Manually',
-                    'Keep list/channel',
-                    'Keep extractor',
-                    'Preferred',
-                    'Default (best)',
-                ];
-                _opt_choose_from_list_idx( $opt, $choice, $prompt, $list );
-            }
             elsif ( $choice eq "retries" ) {
                 my $prompt = 'Download retries';
                 my $digits = 3;
-                _opt_number_range( $opt, $choice, $prompt, 3 )
+                _opt_number_range( $opt, $choice, $prompt, $digits );
             }
             elsif ( $choice eq "timeout" ) {
                 my $prompt = 'Connection timeout (s)';
                 my $digits = 3;
-                _opt_number_range( $opt, $choice, $prompt, 3 )
+                _opt_number_range( $opt, $choice, $prompt, $digits );
             }
             elsif ( $choice eq "merge_enabled" ) {
-                my $prompt = 'Enable merge (experimental)';
+                my $prompt = 'Enable merge';
                 my $list = [
                     'NO',
                     'Ask user',
@@ -260,13 +270,13 @@ sub set_options {
                 my $list = [ qw( NO YES ) ];
                 _opt_choose_from_list_idx( $opt, $choice, $prompt, $list );
             }
-            elsif ( $choice eq "merge_format" ) {
-                my $prompt = 'Choose output format';
+            elsif ( $choice eq "merge_ext" ) {
+                my $prompt = 'Output format of merged files';
                 my $list = [ qw( mkv mp4 ogg webm flv ) ];
                 _opt_choose_from_list_value( $opt, $choice, $prompt, $list );
             }
             elsif ( $choice eq "merge_loglevel" ) {
-                my $prompt = 'Merge Verbosity';
+                my $prompt = '\'ffmpeg\' verbosity';
                 my $list = [ qw( error warning info ) ];
                 _opt_choose_from_list_value( $opt, $choice, $prompt, $list );
             }
@@ -279,29 +289,47 @@ sub set_options {
                 ];
                 _opt_choose_from_list_idx( $opt, $choice, $prompt, $list );
             }
-            elsif ( $choice eq "max_info_width" ) {
-                my $prompt = 'Maximum Info width';
-                my $digits = 3;
-                _opt_number_range( $opt, $choice, $prompt, 3 )
+            elsif ( $choice eq "auto_quality" ) {
+                my $prompt = 'Auto quality';
+                my $list = [
+                    'manually',
+                    'keep_list_channel',
+                    'keep_extractor',
+                    'preferred',
+                    'default',
+                ];
+                _opt_choose_from_list_value( $opt, $choice, $prompt, $list );
             }
-            elsif ( $choice eq "max_len_f_name" ) {
-                my $prompt = 'Maximum filename length';
-                my $digits = 3;
-                _opt_number_range( $opt, $choice, $prompt, 3 )
+            elsif ( $choice eq "print_pref_qual" ) {
+                my $pref = read_json( $opt->{preferred_file} );
+                if ( ! %$pref ) {
+                    choose( [ 'Close with ENTER' ], { prompt => 'No preferred qualities set.' } );
+                }
+                else {
+                    my @p = ( ' ' );
+                    for my $key ( sort keys %$pref ) {
+                        push @p, "$key:";
+                        my $c = 1;
+                        push @p, map { sprintf "%4d. [%s]\n", $c++, join ',', @$_ } @{$pref->{$key}};
+                        push @p, ' ';
+                    }
+                    choose( \@p, { prompt => 'Close with ENTER', layout => 3, justify => 0 } );
+                }
             }
-            elsif ( $choice eq "kb_sec_len" ) {
-                my ( $min, $max ) = ( 3, 9 );
-                my $prompt = 'Digits for "k/s" (download speed)';
-                _opt_number( $opt, $choice, $prompt, $min, $max );
+            elsif ( $choice eq "pref_qual_slots" ) {
+                my $prompt = 'Slots \'preferred qualilties\'';
+                my $digits = 2;
+                _opt_number_range( $opt, $choice, $prompt, $digits );
             }
-            elsif ( $choice eq "video_dir" ) {
-                my $prompt = 'Video directory';
-                _opt_choose_a_directory( $opt, $choice, $prompt );
+            elsif ( $choice eq "log_info" ) {
+                my $prompt = 'Enable info-logging';
+                my $list = [ qw( NO YES ) ];
+                _opt_choose_from_list_idx( $opt, $choice, $prompt, $list );
             }
             elsif ( $choice eq "max_channels" ) {
                 my $prompt = 'Channelhistory';
                 my $digits = 3;
-                _opt_number_range( $opt, $choice, $prompt, 3 )
+                _opt_number_range( $opt, $choice, $prompt, $digits );
             }
             elsif ( $choice eq "channel_by_timestamp" ) {
                 my $prompt = 'Sort channel history';
@@ -311,13 +339,9 @@ sub set_options {
                 ];
                 _opt_choose_from_list_idx( $opt, $choice, $prompt, $list );
             }
-            elsif ( $choice eq "list_size_idx" ) {
-                my $prompt = 'Channel/Playlist videos:';
-                my $list = [
-                    'All',
-                    'Latest 50',
-                ];
-                _opt_choose_from_list_idx( $opt, $choice, $prompt, $list );
+            elsif ( $choice eq "video_dir" ) {
+                my $prompt = 'Video directory';
+                _opt_choose_a_directory( $opt, $choice, $prompt );
             }
             elsif ( $choice eq "extractor_dir" ) {
                 my $prompt = 'Use extractor directories';
@@ -333,22 +357,47 @@ sub set_options {
                 ];
                 _opt_choose_from_list_idx( $opt, $choice, $prompt, $list );
             }
-            elsif ( $choice eq "upload_date_is_file_date" ) {
+            elsif ( $choice eq "max_len_f_name" ) {
+                my $prompt = 'Maximum filename length';
+                my $digits = 3;
+                _opt_number_range( $opt, $choice, $prompt, $digits );
+            }
+            elsif ( $choice eq "modify_timestamp" ) {
                 my $prompt = 'Timestamp to upload date';
                 my $list = [ qw( NO YES ) ];
                 _opt_choose_from_list_idx( $opt, $choice, $prompt, $list );
             }
-            elsif ( $choice eq "sort_item" ) {
+            elsif ( $choice eq "list_size_idx" ) {
+                my $prompt = 'Youtube videos per channel/playlist:';
+                my $list = [
+                    'All',
+                    'Latest 50',
+                ];
+                _opt_choose_from_list_idx( $opt, $choice, $prompt, $list );
+            }
+            elsif ( $choice eq "list_sort_item" ) {
                 sort_order_list( $opt );
             }
             elsif ( $choice eq "show_view_count" ) {
                 my $prompt = 'Show view count in channel/playlist menu';
                 my $list = [
-                    'Sort "view count"',
+                    'If sort-"view count"',
                     'Always',
                 ];
                 _opt_choose_from_list_idx( $opt, $choice, $prompt, $list );
             }
+            elsif ( $choice eq "max_info_width" ) {
+                my $prompt = 'Maximum Info width';
+                my $digits = 3;
+                _opt_number_range( $opt, $choice, $prompt, $digits );
+            }
+
+            elsif ( $choice eq "kb_sec_len" ) {
+                my ( $min, $max ) = ( 3, 9 );
+                my $prompt = 'Digits for "k/s" (download speed)';
+                my $digits = 1;
+                _opt_number_range( $opt, $choice, $prompt, $digits );
+            }
             else { die $choice }
         }
     }
@@ -376,15 +425,19 @@ sub _opt_choose_a_directory {
 }
 
 
-sub _local_read_line {
-    my ( $opt, $section, $prompt ) = @_;
-    my $current = $opt->{$section} // '';
+sub _local_readline {
+    my ( $opt, $section, $prompt, $list ) = @_;
     my $trs = Term::ReadLine::Simple->new();
-    # Readline
-    my $string = $trs->readline( $prompt, { default => $current } );
-    $opt->{$section} = $string;
-    $opt->{change}++;
-    return;
+    my $new_list = $trs->fill_form(
+        $list,
+        { prompt => $prompt, auto_up => 2, confirm => '  CONFIRM', back => '  BACK' }
+    );
+    if ( $new_list ) {
+        for my $i ( 0 .. $#$new_list ) {
+            $opt->{$section} = $new_list->[$i][1];
+        }
+        $opt->{change}++;
+    }
 }
 
 
@@ -401,88 +454,107 @@ sub _opt_number_range {
 }
 
 
-sub _opt_number {
-    my ( $opt, $section, $prompt, $min, $max ) = @_;
-    my $current = $opt->{$section};
-    # Choose
-    my $choice = choose(
-        [ undef, $min .. $max ],
-        { prompt => $prompt . ' [' . $current . ']:', layout => 1, justify => 1, order => 0, undef => '<<' }
-    );
-    return if ! defined $choice;
-    $opt->{$section} = $choice;
-    $opt->{change}++;
-    return;
-}
-
-
 sub _opt_choose_from_list_idx {
     my ( $opt, $section, $prompt, $list ) = @_;
-    $prompt .= ' [' . ( $list->[$opt->{$section}] // '--' ) . ']';
-    my @pre = ( undef );
-    my $idx = choose(
-        [ @pre, map { "- $_" } @$list ],
-        { prompt => $prompt, layout => 3, undef => '<<', index => 1 }
-    );
-    if ( ! defined $idx || $idx == 0 ) {
-        return;
+    my $backup = $opt->{$section};
+    my $confirm = '  CONFIRM';
+    my @pre = ( undef, $confirm );
+    my $choices = [ @pre, map { "- $_" } @$list ];
+    while ( 1 ) {
+        my $local_prompt = $prompt . ' [' . ( $list->[$opt->{$section}] // '--' ) . ']';
+        my $idx = choose(
+            $choices,
+            { prompt => $local_prompt, layout => 3, undef => '  BACK', index => 1 }
+        );
+        if ( ! defined $idx || $idx == 0 ) {
+            $opt->{$section} = $backup;
+            return;
+        }
+        if ( $choices->[$idx] eq $confirm ) {
+            $opt->{change}++;
+            return;
+        }
+        $opt->{$section} = $idx - @pre;
     }
-    $opt->{$section} = $idx - @pre;
-    $opt->{change}++;
-    return;
 }
 
 
 sub _opt_choose_from_list_value {
     my ( $opt, $section, $prompt, $list ) = @_;
-    $prompt .= ' [' . ( $opt->{$section} // '--' ) . ']';
-    my @pre = ( undef );
-    my $choice = choose(
-        [ @pre,  map { "- $_" } @$list ],
-        { prompt => $prompt, layout => 3, undef => '<<', index => 0 }
-    );
-    if ( ! defined $choice ) {
-        return;
+    my $backup = $opt->{$section};
+    my $confirm = '  CONFIRM';
+    my @pre = ( undef, $confirm );
+    while ( 1 ) {
+        my $local_prompt = $prompt . ' [' . ( $opt->{$section} // '--' ) . ']';
+        my $choice = choose(
+            [ @pre,  map { "- $_" } @$list ],
+            { prompt => $local_prompt, layout => 3, undef => '  BACK', index => 0 }
+        );
+        if ( ! defined $choice ) {
+            $opt->{$section} = $backup;
+            return;
+        }
+        if ( $choice eq $confirm ) {
+            $opt->{change}++;
+            return;
+        }
+        $choice =~ s/^-\s//;
+        $opt->{$section} = $choice;
     }
-    $choice =~ s/^-\s//;
-    $opt->{$section} = $choice;
-    $opt->{change}++;
-    return;
 }
 
 
 sub sort_order_list {
     my ( $opt ) = @_;
+    my $backup_item  = $opt->{list_sort_item};
+    my $backup_order = $opt->{list_sort_order};
     my $sort_items = [
         { key => 'upload_date',    prompt => 'Upload date' },
         { key => 'title',          prompt => 'Title'       },
         { key => 'view_count_raw', prompt => 'View count'  },
         { key => 'duration',       prompt => 'Duration'    },
     ];
-    my $first_index = first_index { $_->{key} eq $opt->{sort_item} } @$sort_items;
-    my $prompt = sprintf 'Sort order [%s]:', $sort_items->[$first_index]{prompt};
-    my @pre = ( undef );
+    my $confirm = '  CONFIRM';
+    my @pre = ( undef, $confirm );
     my $choices = [ @pre, map( "- $_->{prompt}", @$sort_items ) ];
-    my $idx = choose (
-        $choices,
-        { prompt => $prompt, layout => 3, index => 1, undef => '<<' }
-    );
-    if ( ! defined $idx || ! defined $choices->[$idx] ) {
-        return;
+    my $idx;
+
+    while ( 1 ) {
+        my $first_index = first_index { $_->{key} eq $opt->{list_sort_item} } @$sort_items;
+        my $prompt = sprintf 'Sort item [%s]:', $sort_items->[$first_index]{prompt};
+        $idx = choose (
+            $choices,
+            { prompt => $prompt, clear_screen => 0, layout => 3, index => 1, undef => '  BACK' }
+        );
+        if ( ! defined $idx || ! defined $choices->[$idx] ) {
+            $opt->{list_sort_item} = $backup_item;
+            return;
+        }
+        if ( $choices->[$idx] eq $confirm ) {
+            $opt->{change}++;
+            last;
+        }
+        $idx -= @pre;
+        $opt->{list_sort_item} = $sort_items->[$idx]{key};
     }
-    $opt->{change}++;
-    $idx -= @pre;
-    $opt->{sort_item} = $sort_items->[$idx]{key};
-    my $order_prompt = sprintf '%s:', $sort_items->[$idx]{prompt};
-    my $order = choose (
-        [ @pre, 'Desc', 'Asc' ],
-        { prompt => $order_prompt, layout => 1, undef => '<<' }
-    );
-    if ( ! defined $order ) {
-        return;
+
+    while ( 1 ) {
+        my $order_prompt = sprintf 'Sort order %s [%s]:', $sort_items->[$idx]{prompt}, $opt->{list_sort_order};
+        my $choice = choose (
+            [ @pre, '- Desc', '- Asc' ],
+            { prompt => $order_prompt, clear_screen => 0, layout => 3, undef => '<<' }
+        );
+        if ( ! defined $choice ) {
+            $opt->{list_sort_order} = $backup_order;
+            return;
+        }
+        if ( $choice eq $confirm ) {
+            $opt->{change}++;
+            return;
+        }
+        $choice =~ s/^-\s//;
+        $opt->{list_sort_order} = $choice;
     }
-    $opt->{sort_order} = $order;
-    return;
 }
 
 
